/*
 * Author: WWS
 *
 * Purpose: annotated version of K&R word reading program (p. 20).
 * illustrates conditional branching, while loop, program design
 * process
 */

#include <stdio.h>

#define IN  1   /* inside a word */
#define OUT 0   /* outside a word */

/* count lines, words, and characters in input */
int main() {

  /**** declarations: ****/
  int c;         /* workspace for char (converted to int) from getchar */
  int nl=0;      /* accumulated number of lines */
  int nw=0;      /* accumulated number of words */
  int nc=0;      /* accumulated number of characters */
  int state=OUT; /* flag indicating in/out of word */
  /**** end of declarations ****/

  /* character reading loop 
   *
   * - the expression (var = function(...))
   * evaluates to the value of var returned by the function - a handy
   * shortcut 
   *
   * - EOF is the end-of-file marker (look it up!), an int
   * indicating the end of input - either the end of a file, or (for
   * keyboard input)
   */
  
  while ((c=getchar()) != EOF) {
    nc++;               /* pre- or post-fix - doesn't matter here (why?) */
    if (c=='\n') nl++;  /* '\n' is the end-of-line character (p. 38) */
    if ((c==' ')  ||
	(c=='\n') ||
	(c=='\t')) 
      state=OUT;        /* white space (space, newline, tab) encountered */
    else {

      /* logic: if you weren't in a word, and the next char encountered
	 is not white space, then you have found another word */
      if (state==OUT) {
	state=IN;
	nw++;
      }
    }
  }

  printf("lines = %d words = %d chars = %d\n",nl,nw,nc);

}
