/*************************************************************************

Copyright Rice University, 2004-2010.
All rights reserved.

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, provided that the above copyright notice(s) and this
permission notice appear in all copies of the Software and that both the
above copyright notice(s) and this permission notice appear in supporting
documentation.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY
RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS
NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL
DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall
not be used in advertising or otherwise to promote the sale, use or other
dealings in this Software without prior written authorization of the
copyright holder.

**************************************************************************/

/*
 * Author: WWS
 * CAAM 420 exception class - mild morph of RVL::RVLException
 *
 * Uses std::string to build up exception messages.
 */

#ifndef __CAAM420_EXCEPT
#define __CAAM420_EXCEPT

#define BUFLEN 100

#include "cppstd.hh"

/** An implementation of the std::exception interface, with additional
    methods so it can be used more like a output stream.
*/

class CAAM420Exception: public std::exception {

private:
  string msg;

public:

  CAAM420Exception(): msg("") {}
  CAAM420Exception(const CAAM420Exception & s): msg("") { msg +=s.msg; }
  virtual ~CAAM420Exception() throw() {}

  // report function - from std::exception
  const char* what() const throw() { return msg.c_str(); }

  CAAM420Exception & operator<< ( string str ) { 
    msg += str;
    return *this;
  }
  CAAM420Exception & operator<< ( const char* str ) { 
    msg += str;
    return *this;
  }
  CAAM420Exception & operator<< ( int i ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%d", i );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( unsigned int i ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%u", i );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( long i ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%ld", i );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( unsigned long i ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%lu", i );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( short i ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%d", i );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( unsigned short i ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%d", i );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( double d ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%g", d );
    msg += buf;
    return *this;
  }
  CAAM420Exception & operator<< ( float d ) {
    char buf[ BUFLEN ];
    sprintf( buf, "%g", d );
    msg += buf;
    return *this;
  }
  template<class T>
  CAAM420Exception & operator<< ( complex<T> d ) {
    char buf[ BUFLEN ];
    sprintf( buf, "(%g,%g)", d.real(),d.imag() );
    msg += buf;
    return *this;
  }

  CAAM420Exception & operator<< ( char c ) {
    char buf[ BUFLEN ];
    buf[ 0 ] = c;
    buf[ 1 ] = '\0';
    msg += buf;
    return *this;
  }

};



#endif
