/*
 * Author: WWS
 * Purpose: simple linear operator base class
 */

#ifndef __CAAM420_MAT_T_BASE__
#define __CAAM420_MAT_T_BASE__

#include "vectbase.hh"

/** basic interface template for linear operators = implicit
    matrices - chief attribute is ability to multiply a vector.
*/
template<typename Scalar>
class MatTBase {

public:

  virtual ~MatTBase() {}

  /** abstract nrows, ncols functions */
  virtual int getNumRows() const = 0;
  virtual int getNumCols() const = 0;

  /** abstract indexing pair */
  virtual Scalar & operator()(int,int) = 0;
  virtual Scalar const & operator()(int,int) const = 0;

  /** main operator - given a default implementation using
      the abstract interfaces of this and VecPPBase. Very much
      same as old matmult, except for dim check
  */
  virtual void matmult(VecTBase<Scalar> const & vecin,
		       VecTBase<Scalar> & vecout) const {
    if (vecin.getLength() != this->getNumCols()) {
      cerr<<"Error: MatTBase::matmult\n";
      cerr<<"input vector length = "<<vecin.getLength()<<"\n";
      cerr<<"number of cols in this = "<<this->getNumCols()<<"\n";
      exit(1);
    }
    if (vecout.getLength() != this->getNumRows()) {
      cerr<<"Error: MatTBase::matmult\n";
      cerr<<"output vector length = "<<vecout.getLength()<<"\n";
      cerr<<"number of rowss in this = "<<this->getNumRows()<<"\n";
      exit(1);
    }
    vecout.zero();
    for (int i=0;i<this->getNumRows(); i++) {
      for (int j=0;j<this->getNumCols();j++) {
	vecout(i)+=(*this)(i,j)*vecin(j);
      }
    }
  }

  /** reporting function - shows C++ stream equivalents of format
      conversion - "%n.me" in fprintf has same effect as
      str<<setw(n)<<setprecision(m)<<setiosflags(ios::scientific)
   */
     
  virtual ostream & report(ostream & str) const = 0;

};

/** template for linear operator objects with solver method
 */
template<typename Scalar>
class MatTSolve: public MatTBase<Scalar> {

public:
  
  virtual ~MatTSolve() {}

  /** solver method */
  virtual void matSolve(VecTBase<Scalar> const & rhs, VecTBase<Scalar> & sol) = 0;

};

#endif
