/*
 * Author: WWS
 * Purpose: simple dense matrix class template, after struct mat
 */

#ifndef __CAAM420_MAT_T_DENSE__
#define __CAAM420_MAT_T_DENSE__

#include "mattbase.hh"
#include "except.hh"

template<typename Scalar>
class MatTDense: public MatTBase<Scalar> {

private:

  int nrows;
  int ncols;
  Scalar * data;

  MatTDense();

public:

  /** the real constructor */
  MatTDense(int n, int m) {
    try {
      nrows=n;
      ncols=m;
      data=NULL;
      data=new Scalar[nrows*ncols];
    }
    catch (bad_alloc) {
      CAAM420Exception e;
      e<<"ERROR: MatTDense constructor\n";
      e<<"failed to allocate "<<nrows*ncols<<" words in data array\n";
      throw e;
    }
  }

  /** copy constructor - deep copy, copies data as well as structure */
  MatTDense(MatTDense<Scalar> const & m) {
    try {
      nrows=m.nrows;
      ncols=m.ncols;
      data=NULL;
      data=new Scalar[nrows*ncols];
      for (int i=0;i<nrows*ncols;i++) data[i]=m.data[i];
    }
    catch (bad_alloc) {
      CAAM420Exception e;
      e<<"ERROR: MatTDense copy constructor\n";
      e<<"failed to allocate "<<nrows*ncols<<" words in data array\n";
      throw e;
    }
  }

  /** the real destructor */
  ~MatTDense() { delete [] data; }

  /** abstract nrows, ncols functions */
  int getNumRows() const { return nrows; }
  int getNumCols() const { return ncols; }

  /** indexing op - non-const*/
  Scalar & operator()(int i,int j) {
    if (i<0 || i>nrows-1) {
      CAAM420Exception e;
      e<<"Error: MatTDense::operator() (non-const)\n";
      e<<"row index = "<<i<<" out of range [0,"<<nrows-1<<"]\n";
      throw e;
    }
    if (j<0 || j>ncols-1) {
      CAAM420Exception e;
      e<<"Error: MatTDense::operator() (non-const)\n";
      e<<"col index = "<<j<<" out of range [0,"<<ncols-1<<"]\n";
      throw e;
    }
    return data[j+ncols*i];
  }

  /** indexing op - const */
  virtual Scalar const & operator()(int i,int j) const {
    if (i<0 || i>nrows-1) {
      CAAM420Exception e;
      e<<"Error: MatTDense::operator() (const)\n";
      e<<"row index = "<<i<<" out of range [0,"<<nrows-1<<"]\n";
      throw e;
    }
    if (j<0 || j>ncols-1) {
      CAAM420Exception e;
      e<<"Error: MatTDense::operator() (const)\n";
      e<<"col index = "<<j<<" out of range [0,"<<ncols-1<<"]\n";
      throw e;
    }
    return data[j+ncols*i];
  }

  ostream & report(ostream & str) const {
    str<<"MatTDense - dense matrix object\n";
    for (int i=0;i<this->getNumRows();i++) {
      for (int j=0;j<this->getNumCols();j++) {
	str<<setw(14)<<setprecision(6)<<setiosflags(ios::scientific)<<(*this)(i,j);
      }
      str<<"\n";
    }
    return str;    
  }
};

#endif
