#ifndef __CAAM420_VECTBASE__
#define __CAAM420_VECTBASE__

#include "except.hh"
#include "cppstd.hh"

/*
 * Author: WWS
 *
 * Purpose: Vector base class templated on scalar type
 */

/* standard class template syntax
 */
template<typename Scalar>
class VecTBase {

public:

  virtual ~VecTBase() {}
  virtual int getLength() const = 0;
  virtual Scalar & operator()(int i) = 0;
  virtual Scalar const & operator()(int i) const = 0;

  virtual void axpy(Scalar a, VecTBase<Scalar> const & x) {
    if (x.getLength() != this->getLength()) {
      CAAM420Exception e;
      e<<"Error: VecTBase::axpy\n";
      e<<"incompatible vectors:\n";
      e<<"length of this = "<<this->getLength()<<"\n";
      e<<"length of that = "<<x.getLength()<<"\n";
      throw e;
    }
    for (int i=0;i<getLength();i++) (*this)(i)+=a*x(i);
  }

  /*
   * needs modification to accomodate complex types
   */
  virtual Scalar dot(VecTBase<Scalar> const & x) const {
    if (x.getLength() != this->getLength()) {
      CAAM420Exception e;
      e<<"Error: VecTBase::dot\n";
      e<<"incompatible vectors:\n";
      e<<"length of this = "<<this->getLength()<<"\n";
      e<<"length of that = "<<x.getLength()<<"\n";
      throw e;
    }
    Scalar d=0.0;
    for (int i=0;i<getLength();i++) d += ((*this)(i))*(x(i));
    return d;
  }

  /* 
   * Another issue - what iz zero? Is 0.0 always right? What about
   * complex?
   */
  virtual void zero() {
    for (int i=0;i<this->getLength();i++) (*this)(i)=0.0;
  }

  virtual ostream & report(ostream & str) const = 0;

};

#endif


