#ifndef __CAAM420_VECTDENSE__
#define __CAAM420_VECTDENSE__

#include "vectbase.hh"

/*
 * Author: WWS
 *
 * Purpose: templated dense vector class 
 */

// note: outside of class declaration, must refer to class by 
// full name - includes template!
template<typename Scalar>
class VecTDense: public VecTBase<Scalar> {

private:

  int len;
  Scalar * vdata;

  // disabled default and copy constructors
  VecTDense();
  VecTDense(VecTDense<Scalar> const &);

public:

  /* main constructor - note use of default arg */
  VecTDense(int _len, bool init=false): VecTBase<Scalar>() {
    len=_len;
    vdata=new Scalar[len];
    if (init) {
      for (int i=0;i<len;i++) vdata[i]=0.0;
    }
  }

  /* destructor - note that call to delete is safe. At moment contemplate
     no subclasses, so need not be virtual.
  */
  ~VecTDense() {
    if (vdata) delete [] vdata;
  }

  int getLength() const { return len; }

  /** inserted to permit efficient const & non-const access to data,
      without intervening function calls which cost significant number of
      cycles per access
   */
  Scalar * getData() { return vdata; }
  const Scalar * getData() const { return vdata; }

  Scalar & operator()(int i) { 
    if (vdata && i >= 0 && i <= len-1) return vdata[i];
    else {
      CAAM420Exception e;
      e<<"Error: VecTDense::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"out of range\n";
      throw e;
    }
  }

  Scalar const & operator()(int i) const {
    if (vdata && i >= 0 && i <= len-1) return vdata[i];
    else {
      CAAM420Exception e;
      e<<"Error: VecTDense::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"out of range\n";
      exit(1);
    }
  }

  /* mandatory report function */
  virtual ostream & report(ostream & str) const {
    str<<"VecTDense object: length="<<len<<"\n";
    for (int i=0;i<len;i++) {
      str<<"v["<<i<<"]="<<vdata[i]<<"\n";
    }
    return str;
  }

};

#endif


