#ifndef __CAAM420_VECTSPARSE__
#define __CAAM420_VECTSPARSE__

#include "vectbase.hh"

template<typename Scalar>
class VecTSparse : public VecTBase<Scalar> {

private:
  int len;
  int nnz;
  int *indices;
  Scalar *data;
  Scalar zip;

  VecTSparse();
  VecTSparse(VecTSparse<Scalar> const &);

public:

  // main constructor for empty sparse vector 
  VecTSparse(int in_dim) : VecTBase<Scalar>() {
    len=in_dim;
    nnz = 0;
    indices = NULL;
    data = NULL;
    zip=0.0;
  }

  // destructor
  ~VecTSparse() {
    if (data) delete [] data;
  }

  int getLength() const { return len; }

  // this returns a mutable (assignable) reference
  // references have to refer to something that
  // exists - so construct entry if not already constructed
  // morph of implementation of set function 
  //   void set(int index, Scalar val){
  // in last week's class
  // checks for existing entry and
  // extends storage for new entries
  Scalar & operator()(int index) { 

    if (index<0 || index > this->getLength()) {
      CAAM420Exception e;
      e<<"Error: VecTSparse::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"index = "<<i<<" out of range [0, "<<this->getLength()<<"\n";
      throw e;
    }

    int i;

    for(i=0;i<nnz;++i){
      if(indices[i] == index){
	return data[i];
      }
    }
    
    // otherwise extend list by one
    // oops - realloc is not appropriate here 
    ++nnz;
    Scalar *tmpdata = new Scalar[nnz];
    for(i=0;i<nnz-1;++i) tmpdata[i] = data[i];

    delete [] data;
    data = tmpdata;

    // oops need to realloc indices
    int *tmpindices = new int[nnz];
    for(i=0;i<nnz-1;++i) tmpindices[i] = indices[i];
    tmpindices[i] = index;
    
    delete [] indices;
    indices = tmpindices;

    return data[nnz-1];
  }

  // implementation of operator () const
  Scalar const & operator()(int index) const {
    
    if (index<0 || index > this->getLength()) {
      CAAM420Exception e;
      e<<"Error: VecTSparse::operator() (const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"index = "<<i<<" out of range [0, "<<this->getLength()<<"\n";
      throw e;
    }

    for(int i=0;i<nnz;++i){
      if(indices[i] == index){
	return data[i];
      }
    }

    return zip;
  }

  // reimplementation of zero operator - minimal storage (none!)
  // note that there is NO MENTION of Scalar 0.0! That's implicit
  // in the indexing operator implementation - and these operators
  // provide the only access to the data vector!!!
  void zero() {
    nnz=0;
    if (data) delete [] data;
    data=NULL;
  }

  // output stored entries
  ostream & report(ostream & str) const {
    for(int i=0;i<nnz;++i)
      str<<"nnz: "<<i<<" index: "<<indices[i]<<" val: "<<data[i]<<"\n";
    return str;
  }

};

#endif
