#ifndef __CAAM420_VECTVEC__
#define __CAAM420_VECTVEC__

#include "vectbase.hh"

/*
 * Author: WWS
 *
 * Purpose: templated dense vector class 
 */

// note: outside of class declaration, must refer to class by 
// full name - includes template!
template<typename Scalar>
class VecTVec: public VecTBase<Scalar> {

private:

  std::vector<Scalar> v;

  // disabled default and copy constructors
  VecTVec();
  VecTVec(VecTVec<Scalar> const &);

public:

  /* main constructor - note use of default arg */
  VecTVec(int len=0, bool init=false): VecTBase<Scalar>(): v(len) {
    if (init) {
      for (int i=0;i<v.size();i++) v[i]=0.0;
    }
  }

  /* destructor - note that call to delete is safe. At moment contemplate
     no subclasses, so need not be virtual.
  */
  ~VecTVec() {}

  int getLength() const { return v.size(); }

  Scalar & operator()(int i) { 
    try {
      return v.at(i);
    }
    catch (out_of_range) {
      CAAM420Exception e;
      e<<"Error: VecTVec::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"index "<<i<<" out of range [0,"<<v.size()<<"\n";
      throw e;
    }      
  }

  Scalar const & operator()(int i) const {
    try {
      return v.at(i);
    }
    catch (out_of_range) {
      CAAM420Exception e;
      e<<"Error: VecTVec::operator() (non-const)\n";
      e<<"attempted access failed - either not initialized or \n";
      e<<"index "<<i<<" out of range [0,"<<v.size()<<"\n";
      throw e;
    }      
  }

  /* mandatory report function */
  ostream & report(ostream & str) const {
    str<<"VecTVec object: length="<<v.size()<<"\n";
    for (int i=0;i<v.size();i++) {
      str<<"v["<<i<<"]="<<[i]<<"\n";
    }
    return str;
  }

  /* add new member at end */
  void push(Scalar a) {
    v.push_back(a);
  }

};

#endif


