#include "op.hh"

/* 
 * implementation of simplest version of Conjugate Gradient
 * iteration, using only base class matrix and vector methods
 */
void cgop(Op const & A,          // linear operator
	  VecPPBase const & b,   // right-hand side
	  VecPPBase & x,         // estimated solution
	  VecPPBase & r,         // residual b-A*x - workspace
	  VecPPBase & p,         // search direction - workspace
	  VecPPBase & ap,        // A*p - workspace
	  int nit) {             // number of iterations (?)
  
  /* workspace */
  double rtr;
  double ptap;
  double alpha;
  double beta;
  int it;

  /* compute the initial residual - note that this
     amounts to a copy, maybe we should add a copy method to 
     the VecPP base class
  */
  r.zero();
  r.axpy(1.0,b);
  
  /* rtr is dot product of r with itself */
  rtr = r.dot(r);
  cout<<"initial squared residual = "<<rtr<<"\n";

  /* initial zearch direction is same as initiali residual -
     another argument for copy method 
  */
  p.zero();
  p.axpy(1.0,r);

  /* initial guess at solution: zero */
  x.zero();

  /* CG loop */
  for (it=0;it<nit;it++) {

    A.matmult(p,ap);
    ptap=p.dot(ap);
    // this is dangerous!!!
    alpha=rtr/ptap;
    x.axpy(alpha,p);
    // r = b-A*x always!!
    r.axpy(-alpha,ap);
    beta=rtr;
    rtr=r.dot(r);
    cout<<"squared residual at step "<<it+1<<" = "<<rtr<<"\n";
    // this is dangerous too!!!
    beta=rtr/beta;
    // a convoluted way of writing p = r + beta*p
    p.axpy(beta-1.0,p);  // p = beta*p
    p.axpy(1.0,r);       // p = r + p

  }
}
