#include "cs.h"

#define MAXC 128

void crsInfoConstruct(CRSINFO * m) {
  m->nr=0;
  m->nc=0;
  m->nnz=0;
  m->inrows=NULL;
  m->incols=NULL;
  m->indata=NULL;
}

void crsInfoDestroy(CRSINFO * m) {
  if (m->inrows) free(m->inrows);
  if (m->incols) free(m->incols);
  if (m->indata) free(m->indata);
  crsInfoConstruct(m);
}
void crsConstruct(CRS * m) {
  m->nr=0;
  m->nc=0;
  m->nnz=0;
  m->ja=NULL;
  m->ia=NULL;
  m->data=NULL;
}

void crsDestroy(CRS * m) {
  if (m->ja) free(m->ja);
  if (m->ia) free(m->ia);
  if (m->data) free(m->data);
  crsConstruct(m);
}

int crsInfoRead(CRSINFO * m, FILE * fp) {

  int i,j;
  int ok1=0;
  int ok2=0;
  int ok3=0;
  int okin1=0;
  int okin2=0;
  int okin3=0;
  char s[MAXC];
  char t[MAXC];

  // avoid memory leaks
  crsInfoDestroy(m);

  while (fgets(s,MAXC,fp)) {
    //    printf("s=%s ok1=%d ok2=%d ok3=%d okin1=%d okin2=%d okin3=%d\n",s,ok1,ok2,ok3,okin1,okin2,okin3); 
    i=0;
    while ((ok1!=1) && i<MAXC-1) {
      ok1=sscanf(&(s[i]),"nr=%d",&(m->nr));
      //      printf("nrows: %d %d %d %s\n",i,ok1,*nrows,&(s[i]));
      i++;
    }
    i=0;
    while ((ok2!=1) && i<MAXC-1) {
      ok2=sscanf(&(s[i]),"nc=%d",&(m->nc));
      //      printf("ncols: %d %d %d %s\n",i,ok2,*ncols,&(s[i]));
      i++;
    }
    i=0;
    while ((ok3!=1) && i<MAXC-1) {
      ok3=sscanf(&(s[i]),"nnz=%d",&(m->nnz));
      //      printf("ncols: %d %d %d %s\n",i,ok2,*ncols,&(s[i]));
      i++;
    }
    i=0;
    while ((okin1!=1) && i<MAXC-1) {
      memset((void *)t,'\0',MAXC);
      okin1=sscanf(&(s[i]),"inrows=%s",t);
      //      printf("in: %d %d %s %s\n",i,okin,in,&(s[i]));
      m->inrows = (char *)malloc((strlen(t)+1)*sizeof(char));
      strcpy(m->inrows,t);
      i++;
    }
    i=0;
    while ((okin2!=1) && i<MAXC-1) {
      memset((void *)t,'\0',MAXC);
      okin2=sscanf(&(s[i]),"incols=%s",t);
      //      printf("in: %d %d %s %s\n",i,okin,in,&(s[i]));
      m->incols = (char *)malloc((strlen(t)+1)*sizeof(char));
      strcpy(m->incols,t);
      i++;
    }
    i=0;
    while ((okin3!=1) && i<MAXC-1) {
      memset((void *)t,'\0',MAXC);
      okin3=sscanf(&(s[i]),"indata=%s",t);
      //      printf("in: %d %d %s %s\n",i,okin,in,&(s[i]));
      m->indata = (char *)malloc((strlen(t)+1)*sizeof(char));
      strcpy(m->indata,t);
      i++;
    }
  }

  if (!(ok1*ok2*ok3*okin1*okin2*okin3)) {
    fprintf(stderr,"Error: crsInfoRead\n");
    fprintf(stderr,"failed to read all necessary data\n");
    return 1;
  }

  if ((m->nr<1) ||
      (m->nc<1) ||
      (m->nnz<1) ||
      (strlen(m->inrows)<1) ||
      (strlen(m->incols)<1) ||
      (strlen(m->indata)<1)) {
    fprintf(stderr,"Error: crsInfoRead\n");
    fprintf(stderr,"failed to assign legit vals\n");
    return 1;
  }

  return 0;
}

void crsInfoWrite(CRSINFO a, FILE * fp) {	  
  fprintf(fp,"nr=%d\n",a.nr);     
  fprintf(fp,"nc=%d\n",a.nc);
  fprintf(fp,"nnz=%d\n",a.nnz);
  fprintf(fp,"inrows=%s\n",a.inrows);
  fprintf(fp,"incols=%s\n",a.incols);
  fprintf(fp,"indata=%s\n",a.indata);
}

int crsRead(CRS * m, CRSINFO a) {
  
  FILE * fp; // input stream

  /* prepare CRS struct */
  crsDestroy(m);

  /* transfer integer data */
  m->nc = a.nc;
  m->nr = a.nr;
  m->nnz = a.nnz;

  /* read row info */
  if (!(fp=fopen(a.inrows,"r"))) {
    fprintf(stderr,"ERROR: crsRead\n");
    fprintf(stderr,"failed to open file %s\n",a.inrows);
    return 1;
  }
  m->ia=(int *)malloc((m->nr+1)*sizeof(int));
  if (m->nr != fread(m->ia,sizeof(int),m->nr,fp)) {
    fprintf(stderr,"ERROR: crsRead\n");
    fprintf(stderr,"failed to read %d ints from file %s\n",a.nr,a.inrows);
    return 1;
  }
  /* last entry always... */
  m->ia[m->nr]=m->nnz;
  fclose(fp);

  /* read col info */
  if (!(fp=fopen(a.incols,"r"))) {
    fprintf(stderr,"ERROR: crsRead\n");
    fprintf(stderr,"failed to open file %s\n",a.incols);
    return 1;
  }
  m->ja=(int *)malloc(m->nnz*sizeof(int));
  if (m->nnz != fread(m->ja,sizeof(int),m->nnz,fp)) {
    fprintf(stderr,"ERROR: crsRead\n");
    fprintf(stderr,"failed to read %d ints from file %s\n",a.nnz,a.incols);
    return 1;
  }
  fclose(fp);

  /* read data */
  if (!(fp=fopen(a.indata,"r"))) {
    fprintf(stderr,"ERROR: crsRead\n");
    fprintf(stderr,"failed to open file %s\n",a.indata);
    return 1;
  }
  m->data=(float *)malloc(m->nnz*sizeof(float));
  if (m->nnz != fread(m->data,sizeof(float),m->nnz,fp)) {
    fprintf(stderr,"ERROR: crsRead\n");
    fprintf(stderr,"failed to read %d floats from file %s\n",a.nnz,a.indata);
    return 1;
  }
  fclose(fp);
  return 0;
}

void crsPrint(CRS m, FILE * fp) {
  int i;
  fprintf(fp,"\n**** CRS struct ****\n");
  fprintf(fp,"\nDim info:\n");
  fprintf(fp,"nr=%d\n",m.nr);     
  fprintf(fp,"nc=%d\n",m.nc);
  fprintf(fp,"nnz=%d\n",m.nnz);
  fprintf(fp,"\nRow pointer array (IA):\n");
  for (i=0;i<m.nr+1;i++) {
    if (i<m.nr) fprintf(fp,"row %d starts at position %d\n",i,m.ia[i]);
    if (i==m.nr) fprintf(fp,"index beyond last = nnz = %d\n",m.ia[i]);
  }
  fprintf(fp,"\nCol index and data arrays (JA, A):\n");
  for (i=0;i<m.nnz;i++) fprintf(fp,"nonzero %d = %12.6e resides in col %d\n",i,m.data[i],m.ja[i]);
}

void crsMatPrint(CRS m, FILE *fp) {
  int i,j,k;             // counters
  float aaa;             // workspace for item to be printed
  // loop over rows
  for (i=0;i<m.nr;i++) {
    // loop over colums
    for (j=0;j<m.nc;j++) {
      // loop over nonzeros in current row
      // to find nonzero in this col, if any
      // and store in workspace
      aaa=0.0f;
      for (k=m.ia[i];k<m.ia[i+1];k++) {
	if (j==(m.ja[k])) aaa=m.data[k];
      }
      // print workspace
      fprintf(fp,"%14.6e ",aaa);
    }
    // line break for new row
    fprintf(fp,"\n");
  }
}

int crsMatVec(CRS m, float * x, float * b) {}

