#ifndef __XWAVE_CS__
#define __XWAVE_CS__

#include "cstd.h"

/*----------------------------------------------------------------------
 * compressed band storage with i/o functions 
 *----------------------------------------------------------------------*/

typedef struct {
  int nr;       // number of rows
  int nc;       // number of columns
  int nnz;      // number of nonzeros
  int * ja;     // column indices of first nonzero in row - length nr
  int * ia;     // index in storage array of first nonzero in row - length nr+1
} CBS;

typedef struct {
  CBS a;
  char * in;
} CBSINFO;

int cbsConstruct(CBS * m);
int cbsDestroy(CBS * m);
int cbsRead(CBSINFO * m, FILE * stream);
int cbsWrite(CBSINFO m, FILE * stream);    
int cbsMatVec(CBS m, float * a, float * x, float * b);
 
typedef struct {
  int nr;       // number of rows
  int nc;       // number of columns
  int nnz;      // number of nonzeros
  int * ja;     // column indices of nonzeros - length nr
  int * ia;     // index in storage array of first nonzero in row - length nr+1
  float * data; // length nnz
} CRS;

typedef struct {
  int nr;
  int nc;
  int nnz;
  char * inrows;
  char * incols;
  char * indata;
} CRSINFO;

void crsInfoConstruct(CRSINFO *);
void crsInfoDestroy(CRSINFO *);
void crsConstruct(CRS *);
void crsDestroy(CRS *);
int crsInfoRead(CRSINFO * m, FILE *);
void crsInfoWrite(CRSINFO m, FILE *);    
int crsRead(CRS *, CRSINFO);
void crsPrint(CRS, FILE *);
void crsMatPrint(CRS, FILE *);
//void crsWrite(CRS, CRSINFO);    
int crsMatVec(CRS, float *, float *);

#endif
