/* Comments in C look like this */

/* and can go on for multiple lines
   like this 
*/

/* USE LOTS OF COMMENTS!!! BUT make them meaningful */

/* 
   ex0.c
   William W. Symes
   Annotated version of K&R's "hello, world" 
*/

/* The C compiler turns a text file into an program unit in machine
   language, in several stages. One of the first stages runs the C
   preprocessor (cpp), which turns lines beginning with "#" into other
   text in various ways. The most important cpp command is "#include
   ...", which causes the file whose name appears after "include" to
   be included in the text.

   In order to use the printf library function, you must include the
   header file (stdio.h) that contains its declaration - declarations of
   functions and variables in C MUST precede their use */

#include <stdio.h>

/* all program units in C are functions, even the main program - it
   returns an int. K&R do not include the "int" declaration for the
   return value in their example codes, but you should. Your programs
   will compile without it, but the source code is actually
   incomplete. Leaving off the return value is an inheritance from the
   earlier non-standard (K&R 1st edn) C. */

main() {

  /* using printf (Ch. 1, also pp. 154 ff in K&R)
     return type: int
     argument list: format conversion string containing format 
     conversion instructions and string constants, then list of
     variables and constants to be printed. 
     Note: if you want a CR, include "\n" in the conversion string !!! */


  /* first version: printf can be called with just the conversion string,
     if there are no items that need formatting (numbers, characters...)
  */

  printf("hello, world\n");

  /* second version: "hello, world" is a string (basically an ordered
     collection of characters - more on the precise defn later", and
     printf knows how to format it. The conversion character for
     formatting strings is "s". 
  */
  printf("%s\n","hello, world");

  /* proof that printf is a function: print the int returned by
     printf. The conversion character for an int is "d". Exactly what
     int is returned?  (This is a meaningful comment)
  */
  printf("%d\n",printf("hello, world\n"));

  /* you can print integer constants the same way */
  printf("%d\n",13);

  /* the declaration of main implies that it returns an int. Which
     int?  If you don't say which, then the result is not under your
     control. "Normal return" in the OS is indicated by the int
     0. Your main function will return control to the OS. So it should
     return 0 if all goes well. We will discuss how to indicate
     not-going-well in future examples. For now we assume that you
     have reached the end of the program with no mishaps, so the final
     statement should specify the "normal return" value. Here is one of the
     two possible ways to specify what a function returns:
  */ 
  return 0;

  /* for main, and main only (a special function, in that it talks to
     the OS rather than to other functions in your program, you could
     also have used

  exit(0);

     which calls the C function exit, which returns control to the OS
     with the value indicated. Either method produces the correct
     result.
  */

  /* end of program (a meaningless or "well, duh!" comment)
  */
}
