/* 
   Author: WWS 

   Purpose: illustrate how to use 1D arrays to fake multidimensional
   arrays via index arithmetic - also matrix multiplication coded this
   way
*/

#include <stdio.h>

int main() {

  const int ncols = 2; /* number of columns */
  const int nrows = 3; /* number of rows    */
  int i,j;             /* loop counters     */
  double a[nrows*ncols]; 
  double b[nrows];
  double x[ncols];

  /* assign matrix entries - store in column-major, i.e. row index is fastest
     for good performance, all loops should be over row index  - note that 
     this is exactly the opposite of C multidim array storage convention!
  */
  for (j=0;j<ncols;j++)
    for (i=0;i<nrows;i++)
      a[i+j*nrows]=i+j;
  for (i=0;i<nrows;i++) {
    for (j=0;j<ncols;j++) printf("a(%d,%d)=%12.4e ",i,j,a[i+j*nrows]);
    printf("\n");
  }
  printf("\n");

  /* define x (input vector), print */
  for (j=0;j<ncols;j++) {
    x[j]=j+1;
    printf("x(%d)=%12.4e\n",j,x[j]);
  }
  printf("\n");

  /* multiply x by a, store in b, print result. For performance,
     access fast index first - build up product as linear comb of
     rows */
  for (i=0;i<nrows;i++) b[i]=0.0;
  for (j=0;j<ncols;j++) 
    for (i=0;i<nrows;i++) 
      b[i]+=a[i+j*nrows]*x[j];
  for (i=0;i<nrows;i++)
    printf("b(%d)=%12.4e\n",i,b[i]);
  
  return 0;
}
