/*
 * Author: WWS
 *
 * Purpose: illustrate some of the "better C" features of C++
 *
 */

/* Like cstd.h. 
 * Common suffixes for C++ header files: .H, .hh, .hpp, and even .h
 */
#include "cppstd.hh"

// reference note the reference syntax in the argument, 
// bool return type
bool sqrtfun2( double &);

int main(int argc, char ** argv) {

  double x; 
    
  // the C++ i/o facility is the subject of an entire chapter
  // (21) in Stroustrup, and mostly concerns its design to handle
  // generalized types (classes). For the part analogous to C's 
  // stdio, a few examples will suffice:
  
  // C:
  // printf("enter a number:\n");
  // C++:
  cout << "enter a number:\n";

  // C: 
  // scanf("%lg",x);
  cin >> x;
  // note that C++ input stream is "smart" about type, inferring it
  // from the target object.

  // it's completely legal to put a declaration anywhere - not just 
  // a language extension
  double y=x;
  /* C:
  if (sqrtfun1(&x)) {
    printf("no sqrts of neg numbers\n");
  }
  else {
    printf("sqrt(%14.6e) = %14.6e\n",y,x);
  }
  */

  // C++ - it's possible to control the field width and precision,
  // see section 21.4. The formatting system is very flexible and 
  // correspondingly complicated - will see some later examples.
  // Fortunately the default widths and precisions for each basic
  // type are usually adequate.
  if (sqrtfun2(x)) {
    cout<<"no sqrts of neg numbers\n";
  }
  else {
    cout<<"sqrt("<<y<<") = "<<x<<"\n";
  }

}
