#include "cppstd.hh"

// the old C convention (int return, zero = normal)
// is still frequently used
// note default assignment of value to last arg
int powr(double & x, int p=1) {
  // don't try to take negagive powers
  if (p<1) return 1;
  double b=x;

  // note declaration of counter within loop scope - 
  // VERY convenient
  for (int i=0;i<p-1;i++) x*=b;
  return 0;
}

int main(int argc, char ** argv) {

  int n;
  cout<<"enter dimension: ";
  cin>>n;

  // can't have an array len < 1
  if (n<1) {
    cerr<<"aaiiieeeeeee!!!!!!!!!\n";
    exit(1);
  }

  /*
   * C++ array allocation: the C equivalent would be
   *
   * double * x = (double *)malloc(n*sizeof(double));
   *
   * note that the C++ operator new "understands" the amount
   * of memory that must be allocated for the type, and does 
   * not need to be "told".
   *
   * operator new can be used to allocate things other than 
   * arrays, as will be seen shortly.
   */
  double * x = new double[n];
  double * y = new double[n];

  for (int i=0;i<n;i++) {
    cout<<"enter a number: ";
    cin >> x[i];
    y[i]=x[i];
  }

  if (argc==2) {
    // not very robust, but will do for this expl
    int p=atoi(argv[1]);
    
    
    for (int i=0;i<n;i++) {
     if (powr(x[i],p)) 
       cout<<" no negative powers\n";
     else
       cout<<p<<"th power of "<<y[i]<<" = "<<x[i]<<"\n";
    }
  }
  else if (argc==1) {
    // note - left off last arg! uses default
    for (int i=0;i<n;i++) {
      powr(x[i]);
      cout<<"you entered "<<x[i]<<"\n";
    }
  }
  else {
    cout<<"Usage: ex47.x [string rep of integer]\n";
  }

  /*
   * analogous to C
   *
   * free(x);
   *
   * NOTE: you cannot mix and match - if you allocate with
   * new, you must deallocate with delete
   *
   * NOTE: to deallocate arrays, the correct syntax includes the
   * empty square brackets
   */
  delete [] x;
  delete [] y;

  exit(0);
}

      
