/*
 * Author: WWS
 *
 * Purpose: extremely simple run-around-the-block for
 * very simple array storage class - after ex40.c
 *
 */

#include "vecppdense.hh"
#include "vecppsparse.hh"

int main(int argc, char ** argv) {

  // for the nth time - note no need to declare counters at the
  // top, or anywhere outside of loop bodies
  int n=10;            // vector size

  // create a VecPP - note, default constructor has no argument
  // no need to give any, so this looks like a declaration - but
  // it also initializes the private data members.
  VecPPDense v(n);

  // now the length is an attribute of the VecPP object, so 
  // identify as such
  for (int i=0;i<v.getLength();i++)    // fill it with floats
    //    v.set(i,(double)i);
    v(i)=(double)i;

  // print it out on C++ version of stdout (like printf)
  cout<<"Dense Vector Report:\n";
  v.report(cout);
  cout<<"\n";

  cout<<"instantiate VecPPSparse object, assign element 5, uses non-const index op\n";
  VecPPSparse w(n);

  w(5)=1.0;

  cout<<"Sparse Vector Report:\n";
  w.report(cout);

  cout<<"get element 6 = "<<w(6)<<" - also invokes non-const op(), since object was not declared const\n";

  cout<<"report again\n";

  w.report(cout);

  cout<<"declare const reference to w\n";
  VecPPSparse const & z = w;

  /*
  cout<<"attempt to assign, thus invoking non-const operator()\n";
  z(1)=2.0;
  */

  cout<<"access element of w through const reference - no change\n";
  cout<<"in w's storage\n";
  cout<<"w(7)="<<w(7)<<"\n";
  w.report(cout);

  cout<<"dot product = "<<v.dot(w)<<" only invokes const op()\n";
  cout<<"either way! dot product = "<<w.dot(v)<<"\n";
  cout<<"note: only base class methods used in the calculation!\n";
  // why is this? discuss

  cout<<"report again\n";

  w.report(cout);

  cout<<"\nadd dense and 10*sparse, store in dense\n";
  cout<<"since addition uses only base class\n";
  cout<<"methods, this should work. Since arg is const, storage of w\n";
  cout<<"is not affected\n";

  v.axpy(10.0,w);

  cout<<"dense output:\n";
  v.report(cout);
  cout<<"sparse input:\n";
  w.report(cout);
  exit(0);
}
