/*
 * Author: WWS
 *
 * Purpose: create matrix data file structure using minfo standard
 * and GSL functions, for further testing.
 */

#include "mio.h"
#include <gsl/gsl_matrix.h>

int main() {
  /* the usual */
  int nrows;
  int ncols;
  /* counters */
  int i,j;

  /* char arrays for data and header files */
  char in[MAXC];
  char hdr[MAXC];

  /* file pointers for header, data files */
  FILE * fp;
  FILE * fpd;

  /* gsl_matrix struct to hold matrix data */
  gsl_matrix * x;

  /* hard-wired header filename */
  if (!(fp=fopen("test.minfo","r"))) {
    fprintf(stderr,"oops... no file %s\n",hdr);
    exit(1);
  }

  /* read using minfo function */
  if (minfo_read(&nrows,&ncols,in,fp)) {
    fprintf(stderr,"oops... can't read file %s\n",hdr);
    exit(1);
  }

  /* allocate gsl matrix */
  if (!(x = gsl_matrix_alloc(nrows,ncols))) {
    fprintf(stderr,"oops... can't alloc gsl_matrix\n");
    exit(1);
  }
  
  /* assign matrix endries - row-major, but this (Hilbert) matrix
     is symmetric!
  */
  for (i=0;i<nrows;i++) {
    for (j=0;j<ncols;j++) {
      gsl_matrix_set(x,i,j,1.0/(double)(i+j+1));
    }
  }

  /* write data file */
  if (!(fpd=fopen(in,"w"))) {
    fprintf(stderr,"oops... no file %s\n",in);
    exit(1);
  }

  /* write out data using gsl function */
  if (gsl_matrix_fprintf(fpd,x,"%14.6e")) {
    fprintf(stderr,"oops... no gsl_matrix printf'd\n");
    exit(1);
  }  

  /* close output unit */
  fclose(fpd);

  /* open input unit */
  if (!(fpd=fopen(in,"r"))) {
    fprintf(stderr,"oops... no file %s\n",in);
    exit(1);
  }

  /* read using gsl function */
  if (gsl_matrix_fscanf(fpd,x)) {
    fprintf(stderr,"oops... no gsl_matrix scanf'd\n");
    exit(1);
  }  

  /* print it out to make sure we've got the right matrix */
  gsl_matrix_fprintf(stdout,x,"%14.6e");

  /* all done */
  exit(0);
}
