/*
 * Author: WWS
 *
 * Purpose: create matrix data file structure using minfo standard
 * and GSL functions, for further testing.
 */

#include "mio.h"
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_linalg.h>

int main() {
  /* the usual */
  int nrows;
  int ncols;
  /* counters */
  int i,j;

  /* char arrays for data and header files */
  char in[MAXC];
  char hdr[MAXC];

  /* file pointers for header, data files */
  FILE * fp;
  FILE * fpd;

  /* gsl_matrix struct to hold matrix data */
  gsl_matrix * a;
  gsl_vector * x;
  gsl_vector * s;
  gsl_vector * b;

  /* permutation and sign */
  gsl_permutation * p;
  int signum;

  /* hard-wired header filename */
  if (!(fp=fopen("test.minfo","r"))) {
    fprintf(stderr,"oops... no file %s\n",hdr);
    exit(1);
  }

  /* read using minfo function */
  if (minfo_read(&nrows,&ncols,in,fp)) {
    fprintf(stderr,"oops... can't read file %s\n",hdr);
    exit(1);
  }

  /* allocate gsl matrix */
  if (!(a = gsl_matrix_alloc(nrows,ncols))) {
    fprintf(stderr,"oops... can't alloc gsl_matrix\n");
    exit(1);
  }
  
  /* open input unit */
  if (!(fpd=fopen(in,"r"))) {
    fprintf(stderr,"oops... no file %s\n",in);
    exit(1);
  }

  /* read using gsl function */
  if (gsl_matrix_fscanf(fpd,a)) {
    fprintf(stderr,"oops... no gsl_matrix scanf'd\n");
    exit(1);
  }  

  /* print it out to make sure we've got the right matrix */
  printf("\n%d x %d Matrix, column-major:\n",nrows,ncols);
  gsl_matrix_fprintf(stdout,a,"%14.6e");

  /* create gsl_vector, assign */
  if (!(x=gsl_vector_alloc(ncols))) {
    fprintf(stderr,"oops... no gsl_vector x\n");
    exit(1);
  }  
  for (i=0;i<ncols;i++) 
    gsl_vector_set(x,i,1.0);

  printf("\n%d Vector, solution of linear system to be created:\n",ncols);
  gsl_vector_fprintf(stdout,x,"%14.6e");

  /* create rhs vector */
  if (!(b=gsl_vector_alloc(nrows))) {
    fprintf(stderr,"oops... no gsl_vector b\n");
    exit(1);
  }  

  for (i=0;i<nrows;i++) 
    gsl_vector_set(b,i,0.0);

  /* assign b=Ax */
  if (gsl_blas_dgemv(CblasNoTrans,1.0,a,x,0.0,b)) {
    fprintf(stderr,"oops... something wrong with GSL matvec\n");
    exit(1);
  }
  printf("\n%d Vector, rhs of linear system:\n",nrows);
  gsl_vector_fprintf(stdout,b,"%14.6e");

  /* ALL OF THIS this should be checked, but let's go merrily on.
   */
  s=gsl_vector_alloc(ncols);
  p = gsl_permutation_calloc(nrows);
  gsl_linalg_LU_decomp(a,p,&signum);
  gsl_linalg_LU_solve(a,p,b,s);

  /* print it out */
  printf("\n%d Vector, computed solution of linear system:\n",ncols);
  gsl_vector_fprintf(stdout,s,"%14.6e");

  /* all done */
  exit(0);
}
