/*
 * Author: WWS
 *
 * Purpose: create matrix data file structure using minfo standard
 * and LAPACK functions, solve linear system
 */

/* declarations for lapack functions - in C++ these would have to 
   be "extern C" - also note the underscore
*/
void dgemv_(char *,      // trans
	    int *,       // m
	    int *,       // n
	    double *,    // alpha
	    double *,    // a
	    int *,       // lda
	    double *,    // x
	    int *,       // incx
	    double *,    // beta
	    double *,    // y
	    int *        // incy
	    );

void dgesv_(int *,       // n
	    int *,       // nrhs
	    double *,    // a
	    int *,       // lda
	    int *,       // ipiv
	    double *,    // b
	    int *,       // ldb
	    int *        // info
	    );

#include "mio.h"

int main() {

  /* the usual */
  int nrows;
  int ncols;
  int n; // nrows = ncols = n

  /* counters */
  int i,j;

  /* char arrays for data and header files */
  char in[MAXC];
  char hdr[MAXC];

  /* file pointers for header, data files */
  FILE * fp;
  FILE * fpd;

  /* double arrays to hold matrix data */
  double * a;
  double * x;
  double * s;
  double * b;

  /* variables needed for lapack calls */
  int * p;
  char trans;
  double alpha;
  double beta;
  int incx;
  int incy;
  int nrhs;
  int info;

  /* hard-wired header filename */
  if (!(fp=fopen("test.minfo","r"))) {
    fprintf(stderr,"oops... no file %s\n",hdr);
    exit(1);
  }

  /* read using minfo function */
  if (minfo_read(&nrows,&ncols,in,fp)) {
    fprintf(stderr,"oops... can't read file %s\n",hdr);
    exit(1);
  }

  /* check for squareness */
  if (nrows != ncols || nrows < 1) {
    fprintf(stderr,"illegal must have square matrix, but\n");
    fprintf(stderr,"nrows=%d ncols=%d\n",nrows,ncols);
    exit(1);
  }
  n=nrows;

  /* allocate arrays */
  a = (double *)malloc(n*n*sizeof(double));
  x = (double *)malloc(n*sizeof(double));
  b = (double *)malloc(n*sizeof(double));
  p = (int *)malloc(n*sizeof(int));

  /* open input unit for matrix */
  if (!(fpd=fopen(in,"r"))) {
    fprintf(stderr,"oops... no file %s\n",in);
    exit(1);
  }

  /* read */
  for (i=0;i<n*n;i++) {
    if (fscanf(fpd,"%lg",&(a[i])) != 1) {
      fprintf(stderr,"oops... no matrix scanf'd\n");
      exit(1);
    }
  }  

  /* print it out to make sure we've got the right matrix */
  printf("\n%d x %d Matrix, column-major:\n",nrows,ncols);
  for (i=0;i<n;i++) {
    for (j=0;j<n;j++) {
      printf("%g ",a[i+j*n]);
    }
    printf("\n");
  }

  /* assign solution vector */
  for (i=0;i<n;i++) x[i]=1.0;
  
  printf("\nSolution of linear system to be created:\n");
  for (i=0;i<n;i++) printf("%g\n",x[i]);

  /* matvec */
  trans='n';
  incx = 1;
  incy = 1;
  beta = 0.0;
  alpha= 1.0;

  dgemv_(&trans,&n,&n,&alpha,a,&n,x,&incx,&beta,b,&incy);

  printf("\nRHS of linear system\n");
  for (i=0;i<n;i++) printf("%g\n",b[i]);

  /* solve system - note that dgesv overwrites b, so if it's
     important you'd want to copy it
  */
  nrhs=1;
  info=0;
  dgesv_(&n,&nrhs,a,&n,p,b,&n,&info);

  printf("\nComputed solution of linear system\n");
  for (i=0;i<n;i++) printf("%g\n",b[i]);

  /* all done */
  free(a);
  free(x);
  free(b);
  free(p);

  exit(0);
  
}
