/*
 * Author: WWS
 * Purpose: simple linear operator base class
 */

#ifndef __CAAM420_MATBASE__
#define __CAAM420_MATBASE__

#include "vecppbase.hh"

/** defines basic interface for linear operators = implicit
    matrices - chief attribute is ability to multiply a vector.
*/
class MatBase {

public:

  virtual ~MatBase() {}

  /** abstract nrows, ncols functions */
  virtual int getNumRows() const = 0;
  virtual int getNumCols() const = 0;

  /** abstract indexing pair */
  virtual double & operator()(int,int) = 0;
  virtual double const & operator()(int,int) const = 0;

  /** main operator - given a default implementation using
      the abstract interfaces of this and VecPPBase. Very much
      same as old matmult, except for dim check
  */
  virtual void matmult(VecPPBase const & vecin,
		       VecPPBase & vecout) const;

  /** reporting function - shows C++ stream equivalents of format
      conversion - "%n.me" in fprintf has same effect as
      str<<setw(n)<<setprecision(m)<<setiosflags(ios::scientific)
   */
     
  virtual ostream & report(ostream & str) const;

};

#endif
