#include "matdensef.hh"

void MatFile::readMat(char const * fname) {

  char in[MAXC];
  int err=0;
  int n;
  int m;
  
  // use C stdio for this part, in order to re-use minfo code
  FILE * fp=fopen(fname,"r");
  if (!fp) {
    cerr<<"Error: MatFile constructor - failed to open "<<fname<<endl;
    exit(1);
  }
  err=minfo_read(&n,&m,in,fp);
  if (err) {
    cerr<<"Error: MatFile constructor - failed to read minfo data from "<<fname<<endl;
    exit(1);
  }
  fclose(fp);
  
  int nrows=this->getNumRows();
  int ncols=this->getNumCols();
  
  if (n!=nrows || m !=ncols) {
    cerr<<"Error: MatFile constructor - nrows, ncols read from "<<fname<<" does not match object data = "<<nrows<<", "<<ncols<<"\n";
    exit(1);
  }
  
  // for this part, use C++ stdio
  ifstream from(in,ios::in);
  int i=0;
  int j=0;
  
  while (i<nrows && from>>(*this)(i,j)) {
    j++;
    if (j==ncols) { j=0; i++; }
  }
  
  // cleanup - in is allocated in 
  from.close();
  
}

void MatFile::writeMat(char const * fname, char * dname) const {

  int nrows=this->getNumRows();
  int ncols=this->getNumCols();
  
  FILE * fp = fopen(fname,"w");
  minfo_write(nrows,ncols,dname,fp);
  fclose(fp);
  
  ofstream to(dname,ios::out);
  for (int i=0;i<nrows;i++) {
    for (int j=0;j<ncols;j++) to<<(*this)(i,j)<<"\n";
  }
  to.close();
}

