/*
 * Author: WWS
 *
 * Purpose: test the GSL Matrix class implementation
 */

#include "matgsl.hh"

int main() {

  /* the usual */
  int nrows;
  int ncols;

  /* char arrays for data and header files */
  char in[MAXC];
  char hdr[MAXC];
  FILE * fp;
  
  /* hard-wired header filename */
  if (!(fp=fopen("test.minfo","r"))) {
    fprintf(stderr,"oops... no file %s\n",hdr);
    exit(1);
  }

  /* read matrix dimensional info using minfo function */
  if (minfo_read(&nrows,&ncols,in,fp)) {
    fprintf(stderr,"oops... can't read file %s\n",hdr);
    exit(1);
  }
  fclose(fp);

  /* define matrix object */
  MatGSL a(nrows,ncols);
  
  /* initialize from file - has correct dims by construction */
  a.readMat("test.minfo");

  /* print it out to make sure we've got the right matrix */
  cout<<"System matrix:\n";
  a.report(cout);

  /* create gsl_vector, assign */
  VecPPGSL x(ncols);
  for (int i=0;i<ncols;i++) x(i)=1.0;
  cout<<"\nSolution vector:\n";
  x.report(cout);

  /* create rhs vector */
  VecPPGSL b(nrows);
  b.zero();

  /* assign b=Ax */
  a.matmult(x,b);
  cout<<"\nRHS of linear system:\n";
  b.report(cout);

  /* Solve system, report computed solution */
  a.matsolve(b,x);

  /* print it out */
  cout<<"\nComputed solution of linear system:\n";
  x.report(cout);

  /* all done */
  exit(0);
}
