/*
 * because everything necessary is already in vec.h,
 * no need for anything else
 */

#include "vec.h"

void vec_construct(struct vec * v) {
  v->len=0;
  v->vdata=NULL;
}

int vec_init(struct vec * v, int n) {
  if ((v->len !=0) || (v->vdata !=0)) {
    fprintf(stderr,"ERROR: vec_init\n");
    fprintf(stderr,"  vector not in initial state - cannot create\n");
    return E_ALLOC;
  }
  if (n<1) {
    fprintf(stderr,"ERROR: vec_init\n");
    fprintf(stderr,"  length requested is %d - must be > 0\n",n);
    return E_BOUNDS;
  }
  v->len=n;
  if (!(v->vdata=(double *)malloc(n*sizeof(double)))) {
    fprintf(stderr,"ERROR: vec_init\n");
    fprintf(stderr,"  failed to allocated %d doubles\n",n);
    return E_ALLOC;
  }

  /* 
   * I inadvertantly left this line off in an earlier revision - 
   * it ran on my mac, probably because the Mac OS-X version of gcc
   * tends to fill uninitialized space with zeroes - and the workspace
   * for the return value was uninitialized! But the Linux version of the
   * same compiler does not do this default initialization, so I was getting
   * the return value of 60 (!), which was just whatever trash happened to
   * sit in the memory assigned for the return value. 
   */
  return 0;
}

void vec_destroy(struct vec * v) {
  if (v->vdata) free(v->vdata);
  vec_construct(v);
}

double vec_get(struct vec v, int i) {
  if (i<0||i>=v.len) {
    fprintf(stderr,"ERROR: vec_get, vector of length %d\n",v.len);
    fprintf(stderr,"  index %d not in range [0, len=%d]\n",i,v.len-1);
    exit(1);
  }
  return v.vdata[i];
}

void vec_set(struct vec v, int i, double x) {
  if (i<0||i>=v.len) {
    fprintf(stderr,"ERROR: vec_set, vector of length %d\n",v.len);
    fprintf(stderr,"  index %d not in range [0, len=%d]\n",i,v.len-1);
    exit(1);
  }
  v.vdata[i]=x;
}

void vec_print(struct vec v) {
  int i;
  printf("Vector structure: dimension=%d\n",v.len);
  for (i=0;i<v.len;i++) 
    printf("%14.6e\n",v.vdata[i]);
}

void vec_fprint(struct vec v,FILE * fp) {
  int i;
  fprintf(fp,"Vector structure: dimension=%d\n",v.len);
  for (i=0;i<v.len;i++) 
    fprintf(fp,"%14.6e\n",v.vdata[i]);
}
