#ifndef __CAAM420_VECPPDENSE__
#define __CAAM420_VECPPDENSE__

#include "vecppbase.hh"

/*
 * Author: WWS
 *
 * Purpose: first cut at dense vector class - essentially a C++
 * port of struct vec.
 */

class VecPPDense: public VecPPBase {

private:

  int len;
  double * vdata;

public:

  /* default constructor 
     NOTE THE PROBLEM POSED BY THIS DEFAULT CONSTRUCTOR-
     IT IS IMPOSSIBLE TO USE THE OBJECT PRODUCED BY IT!!!
  */
  VecPPDense(): VecPPBase() {
    len=0;
    vdata=NULL;
  }

  /* main constructor */
  VecPPDense(int _len, bool init=false): VecPPBase() {
    //    cout<<"VecPPDense constructor\n";
    len=_len;
    vdata=new double[len];
    if (init) {
      for (int i=0;i<len;i++) vdata[i]=0.0;
    }
  }

  /* destructor - note that call to delete is safe. At moment contemplate
     no subclasses, so need not be virtual.

     NOTE proper use of delete for arrays - see Stroustrup p. 251.
  */
  ~VecPPDense() {
    //    cout<<"VecPPDense destructor\n";
    if (vdata) delete [] vdata;
  }

  int getLength() const { return len; }

  double & operator()(int i) { 
    if (vdata && i >= 0 && i <= len-1) return vdata[i];
    else {
      cerr<<"Error: VecPPDense::operator() (non-const)\n";
      cerr<<"attempted access failed - either not initialized or \n";
      cerr<<"out of range\n";
      /* Will see a better solution to this problem */
      exit(1);
    }
  }

  double const & operator()(int i) const {
    if (vdata && i >= 0 && i <= len-1) return vdata[i];
    else {
      cerr<<"Error: VecPPDense::operator() (non-const)\n";
      cerr<<"attempted access failed - either not initialized or \n";
      cerr<<"out of range\n";
      /* Will see a better solution to this problem */
      exit(1);
    }
  }

  /* mandatory report function */
  virtual ostream & report(ostream & str) const {
    str<<"VecPPDense object: length="<<len<<"\n";
    for (int i=0;i<len;i++) {
      str<<"v["<<i<<"]="<<vdata[i]<<"\n";
    }
    return str;
  }

};

#endif


