#ifndef __CAAM420_VECPPSPARSE__
#define __CAAM420_VECPPSPARSE__

// sparse vector class derived from vector base class
class VecPPSparse : public VecPPBase {

private:
  int len;
  int nnz;
  int *indices;
  double *data;
  double zero;

public:

  // main constructor for empty sparse vector 
  // NOTE includes default constructor!!!
  VecPPSparse(int in_dim=0) : VecPPBase() {
    len=in_dim;
    nnz = 0;
    indices = NULL;
    data = NULL;
    zero=0.0;
  }

  // destructor
  ~VecPPSparse() {
    if (data) delete [] data;
  }

  int getLength() const { return len; }

  // this returns a mutable (assignable) reference
  // references have to refer to something that
  // exists - so construct entry if not already constructed
  // morph of implementation of set function 
  //   void set(int index, double val){
  // in last week's class
  // checks for existing entry and
  // extends storage for new entries
  double & operator()(int index) { 

    /************ should put index check here!!! */
    int i;

    for(i=0;i<nnz;++i){
      if(indices[i] == index){
	return data[i];
      }
    }
    
    // otherwise extend list by one
    // oops - realloc is not appropriate here 
    ++nnz;
    double *tmpdata = new double[nnz];
    for(i=0;i<nnz-1;++i) tmpdata[i] = data[i];

    delete [] data;
    data = tmpdata;

    // oops need to realloc indices
    int *tmpindices = new int[nnz];
    for(i=0;i<nnz-1;++i) tmpindices[i] = indices[i];
    tmpindices[i] = index;
    
    delete [] indices;
    indices = tmpindices;

    return data[nnz-1];
  }

  // implementation of operator () const
  double const & operator()(int index) const {
    
    for(int i=0;i<nnz;++i){
      if(indices[i] == index){
	return data[i];
      }
    }

    return zero;
  }

  // reimplementation of zero operator - minimal storage (none!)
  // note that there is NO MENTION of double 0.0! That's implicit
  // in the indexing operator implementation - and these operators
  // provide the only access to the data vector!!!
  void zero() {
    nnz=0;
    if (data) delete [] data;
    data=NULL;
  }

  // output stored entries
  ostream & report(ostream & str) const {
    for(int i=0;i<nnz;++i)
      str<<"nnz: "<<i<<" index: "<<indices[i]<<" val: "<<data[i]<<"\n";
    return str;
  }

};

#endif
