#include "vect.h"

void vec_construct(VEC * v) {
  v->len=0;
  v->vdata=NULL;
}

int vec_init(VEC * v, int n) {
  if ((v->len !=0) || (v->vdata !=0)) {
    fprintf(stderr,"ERROR: vec_create\n");
    fprintf(stderr,"  vector not in initial state - cannot create\n");
    return E_ALLOC;
  }
  if (n<1) {
    fprintf(stderr,"ERROR: vec_create\n");
    fprintf(stderr,"  length requested is %d - must be > 0\n",n);
    return E_BOUNDS;
  }
  v->len=n;
  if (!(v->vdata=(REAL *)malloc(n*sizeof(REAL)))) {
    fprintf(stderr,"ERROR: vec_create\n");
    fprintf(stderr,"  failed to allocated %d REALs\n",n);
    return E_ALLOC;
  }
}

void vec_destroy(VEC * v) {
  if (v->vdata) free(v->vdata);
  vec_construct(v);
}

REAL vec_get(VEC v, int i) {
  if (i<0||i>=v.len) {
    fprintf(stderr,"ERROR: vec_index, vector of length %d\n",v.len);
    fprintf(stderr,"  index %d not in range [0, len=%d]\n",i,v.len-1);
    exit(1);
  }
  return v.vdata[i];
}

void vec_set(VEC v, int i, REAL x) {
  if (i<0||i>=v.len) {
    fprintf(stderr,"ERROR: vec_index, vector of length %d\n",v.len);
    fprintf(stderr,"  index %d not in range [0, len=%d]\n",i,v.len-1);
    exit(1);
  }
  v.vdata[i]=x;
}

void vec_print(VEC v) {
  int i;
  printf("Vector structure: dimension=%d\n",v.len);
  for (i=0;i<v.len;i++) 
    printf("%14.6e\n",v.vdata[i]);
}

void vec_fprint(VEC v,FILE * fp) {
  int i;
  fprintf(fp,"Vector structure: dimension=%d\n",v.len);
  for (i=0;i<v.len;i++) 
    fprintf(fp,"%14.6e\n",v.vdata[i]);
}
