/*
 * Author: WWS
 *
 * Purpose: define simple data type for vector
 * storage, equivalent to GSL block, illustrate
 * uses of typedef
 */


#ifndef __WWS_VECT__
#define __WWS_VECT__

/* 
 * the usual suspects
 */

#include "cstd.h"

/*
 * error flags 
 */

#define E_ALLOC 1
#define E_BOUNDS 2

/* 
 * define a real type and use REAL throughout instead of float or
 * double - that way, you need only switch typedefs IN THIS ONE LINE
 * to change real no. repn, rather than copy the whole code !
 */

typedef double REAL;
//typedef float REAL;

/*
 * an array that knows its length - arguably, a vector
 * uses usual typedef idiom for structs
 */
typedef struct {
  int len;
  REAL * vdata;
} VEC;

/*
 * sets a vector to its initial state
 */
void vec_construct(VEC * v);

/*
 * basic initialization function
 */
int vec_init(VEC * v, int n);

/* 
 * VEC destructor
 */
void vec_destroy(VEC * v);

/*
 * read from an element in the array data member
 */
REAL vec_get(VEC v, int i);

/* 
 * assign to an element in the array data member
 */
void vec_set(VEC v, int i, REAL x);

/* a simple display function */
void vec_print(VEC v);

/* a function to print the vector to a file */
void vec_fprint(VEC v, FILE * fp);

#endif
