#ifndef __CAAM420_VECTDENSE__
#define __CAAM420_VECTDENSE__

#include "vectbase.hh"
extern "C" {
#include <gsl/gsl_vector.h>
}

/*
 * Author: WWS
 *
 * Purpose: templated GSL vector class 
 */

// note: outside of class declaration, must refer to class by 
// full name - includes template!
template<typename Scalar>
class VecTGSL: public VecTBase<Scalar> {
};

template<>
class VecTGSL<double>: public VecTBase<double> {

private:

  gsl_vector * gv;

  // disabled default and copy constructors
  VecTGSL();

public:

  /* main constructor - note use of default arg */
  VecTGSL(int n): VecTBase<double>() { gv = gsl_vector_alloc(n); }

  /* copy constructor */
  VecTGSL(VecTGSL<double> const & v) {
    gv=gsl_vector_alloc(v.getLength());
    gsl_vector_memcpy(gv,v.gv);
  }

  /* destructor - note that call to delete is safe. At moment contemplate
     no subclasses, so need not be virtual.
  */
  ~VecTGSL() { gsl_vector_free(gv); }

  int getLength() const { return (int)(gv->size); }

  double & operator()(int i) { 
    if (i >= 0 && i <= gv->size-1) return gv->data[i];
    else {
      cerr<<"Error: VecTGSL::operator() (non-const)\n";
      cerr<<"attempted access failed - either not initialized or \n";
      cerr<<"out of range\n";
      /* Will see a better solution to this problem */
      exit(1);
    }
  }

  double const & operator()(int i) const {
    if (i >= 0 && i <= gv->size-1) return gv->data[i];
    else {
      cerr<<"Error: VecTGSL::operator() (non-const)\n";
      cerr<<"attempted access failed - either not initialized or \n";
      cerr<<"out of range\n";
      /* Will see a better solution to this problem */
      exit(1);
    }
  }

  /* mandatory report function */
  virtual ostream & report(ostream & str) const {
    str<<"VecTGSL object: length="<<gv->size<<"\n";
    for (int i=0;i<gv->size;i++) {
      str<<"v["<<i<<"]="<<gv->data[i]<<"\n";
    }
    return str;
  }

};

#endif


